/*
	C version of CIO
*/

#include <stdio.h>
#include <stdarg.h>
#include <cio.h>



/*
	cio global variables
*/

IOCB iocb[32];	/*	space for 32 I/O channeles	*/
HTABS htabs[20];	/*	space for 20 I/O devices	*/
static int last_dev = 0;	/*	last device entry in table	*/

static free_iocb(int handle);
static find_free_iocb(void);
static dev_search(register int handle);


int cio(int cmd,...)
{
	int handle;
	int a;	/*	temporary data storage	*/
	register long i;
	register char *p;
	register int (*function)();	/*	handler routine	*/
	va_list argp;	/*	variable arguments	*/

	va_start(argp,cmd);	/*	start of arguments	*/
	switch(cmd)
	{
		case ADD_HANDLER:
			/*
			** This command is used to insert handler vector into HTABS
			** called as: cio(cmd,devname,table)
			*/
			htabs[last_dev].dev_name = va_arg(argp,int);	/*	device name	*/
			htabs[last_dev].vectors = (H_JVEC *)va_arg(argp,long);	/*	address of table	*/
			++last_dev;
			break;
		case OPEN:
			if((handle = find_free_iocb()) == -1)
				return(NO_HANDLES);
			/*
				get open parrameters off of stack
				passed as cio(cmd,name,mode)
			*/
			iocb[handle].command = cmd;
			iocb[handle].dev_name = va_arg(argp,char *);
			iocb[handle].mode = va_arg(argp,int);
			if(( iocb[handle].ichid = dev_search(handle)) < 0)
				return(NO_DEVICE);	/*	could not find device handle	*/
			function = (htabs[iocb[handle].ichid].vectors)->jtab.jumptab[VOPEN];
			(*function)(&iocb[handle],argp);
			return(handle);
			break;
		case CLOSE:
			/*
				get close parrameters off of stack
				passes as: cio(cmd,handle);
			*/
			handle = va_arg(argp,int);
			if(handle > MAX_HANDLE || handle < 0)
				return(INVLD_HNDL);
			if(iocb[handle].ichid < 0)
				return(NOT_OPEN);
			iocb[handle].command = cmd;
			function = (htabs[iocb[handle].ichid].vectors)->jtab.jumptab[VCLOSE];
			(*function)(&iocb[handle],argp);
			return(free_iocb(handle));
		case GETC:
		case GETR:
			/*
			** Get data from handler
			** called as:  cio(cmd,handle,count,buffer)
			** if count == 0, then data returned via function
			** if count > 0, then data returned via buffer
			*/
			handle = va_arg(argp,int);
			if(handle > MAX_HANDLE || handle < 0)
				return(INVLD_HNDL);
			if(iocb[handle].ichid < 0)
				return(NOT_OPEN);
			if((cmd & iocb[handle].mode) == 0)
				return(iocb[handle].status = WRONLY);
			iocb[handle].command = cmd;
			iocb[handle].count = va_arg(argp,long);
			iocb[handle].buffer = va_arg(argp,char *);
			function = (htabs[iocb[handle].ichid].vectors)->jtab.jumptab[VGET];
			if(iocb[handle].count == 0)
			{
				/*
				** buffer count is zero, get data, and exit
				*/
				iocb[handle].status = (*function)(&iocb[handle],argp);
				return(iocb[handle].status);
			}
			else
			{
				/*
				** buffer count > 0, get lots of data
				*/
				i = 0;
				p = iocb[handle].buffer;
				while( ((a = (*function)(&iocb[handle],argp)) > 0) && (i < iocb[handle].count) )
				{
					p[i] = (char)a;
					++i;
					if(!(cmd & 0x02))
					{
						if(a == EOL)
						{
							iocb[handle].status = 0;
							return(i);
						}
					}
				}
				if(!(cmd & 0x02))
					return(iocb[handle].status = TRUNCATED);
				else
					iocb[handle].status = a;
				return(i);
			}
			break;
		case PUTC:
		case PUTR:
			/*
			** Put data to device
			** Called as : cio(cmd,handle,count,buffer)
			**  or
			**           : cio(cmd,handle,0l,data)
			*/
			handle = va_arg(argp,int);
			if(handle > MAX_HANDLE || handle < 0)
				return(INVLD_HNDL);
			if(iocb[handle].ichid < 0)
				return(NOT_OPEN);
			if((iocb[handle].mode & cmd) == 0)
				return(RDONLY);	/*	read only error	*/
			iocb[handle].command = cmd;
			iocb[handle].count = va_arg(argp,long);
			function = (htabs[iocb[handle].ichid].vectors)->jtab.jumptab[VPUT];
			if(iocb[handle].count == 0)
			{
				a = va_arg(argp,int);
				return(iocb[handle].status = (*function)(&iocb[handle],a,argp));
			}
			else
			{
				iocb[handle].buffer = va_arg(argp,char *);
				i = 0;
				p = iocb[handle].buffer;
				while( i < iocb[handle].count)
				{
					a = p[i];
					iocb[handle].status = (*function)(&iocb[handle],a);
					if(cmd == PUTR)
						if(a == EOL)
							return(i);
				}
				if(cmd == PUTR)
					iocb[handle].status = (*function)(&iocb[handle],EOL);
				return(i);
			}
			break;
		case STATUS:
			/*
			** This is where a status call is handled
			**
			** called as: cio(cmd,handle,buffer,count,aux,devname)
			**
			** if handle is -1, then devname is used to open iocb
			** buffer is where status info is put, device dependent
			*/
			handle = va_arg(argp,int);
			if(handle < 0)
			{	/*	this is immeadiate operation	*/
				if((handle = find_free_iocb()) == -1)
					return(NO_HANDLES);	/* open up an IOCB	*/
				iocb[handle].command = cmd;
				iocb[handle].buffer = va_arg(argp,char *);
				iocb[handle].count = va_arg(argp,long);
				iocb[handle].aux = va_arg(argp,int);
				iocb[handle].dev_name = va_arg(argp,char *);
				if(( iocb[handle].ichid = dev_search(handle)) < 0)
					return(NO_DEVICE);	/*	could not find device handle	*/
				function = (htabs[iocb[handle].ichid].vectors)->jtab.jumptab[VSTAT];
				iocb[handle].status = (*function)(&iocb[handle],argp);
				free_iocb(handle);	/*	close iocb	*/
				return(iocb[handle].status);
			}
			else
			{	/*	valid handle for already open iocb	*/
				if(handle > MAX_HANDLE || handle < 0)
					return(INVLD_HNDL);
				iocb[handle].command = cmd;
				iocb[handle].buffer = va_arg(argp,char *);
				iocb[handle].count = va_arg(argp,long);
				iocb[handle].aux = va_arg(argp,int);
				function = (htabs[iocb[handle].ichid].vectors)->jtab.jumptab[VSTAT];
				iocb[handle].status = (*function)(&iocb[handle],argp);
				return(iocb[handle].status);
			}
			break;
		default:
			/*
			** This is where the special commands are handled
			** called as:  cio(cmd,handle,buffer,count,aux,dev_name)
			** Parameters:
			** cmd
			** handle (if -1, then use device name for operation)
			** dev_name (if NULL, use handle number) (long)
			** aux
			** buffer (long)
			** byte_count (long)
			*/
			if((handle = va_arg(argp,int)) < 0)
			{
				/*
					this is an immediate operation
				*/
				if((handle = find_free_iocb()) == -1)
					return(-1);	/* open up an IOCB	*/
				iocb[handle].command = cmd;
				iocb[handle].buffer = va_arg(argp,char *);
				iocb[handle].count = va_arg(argp,long);
				iocb[handle].aux = va_arg(argp,int);
				iocb[handle].dev_name = va_arg(argp,char *);
				if(( iocb[handle].ichid = dev_search(handle)) < 0)	/*	locate handler entry table	*/
					return(-1);	/*	could not find device handle	*/
				function = (htabs[iocb[handle].ichid].vectors)->jtab.jumptab[VSPECIAL];
				iocb[handle].status = (*function)(&iocb[handle],argp);
				free_iocb(handle);
			}
			else
			{
				/*
				** iocb is already open
				*/
				iocb[handle].command = cmd;
				iocb[handle].buffer = va_arg(argp,char *);
				iocb[handle].count = va_arg(argp,long);
				iocb[handle].aux = va_arg(argp,int);
				function = (htabs[iocb[handle].ichid].vectors)->jtab.jumptab[VSPECIAL];
				iocb[handle].status = (*function)(&iocb[handle],argp);				return(iocb[handle].status);
			}
			return(iocb[handle].status);
			break;
	}	/*	end switch cmd	*/
}

static dev_search(register int handle)
{
	/*
		search device table for device named and return index into
		device table
	*/
	register int i;

	for(i=last_dev -1;(i > -1) && (htabs[i].dev_name != *(iocb[handle].dev_name));--i);
	if(i >= 0)
	{
		if(iocb[handle].dev_name[1] == ':')
			iocb[handle].devnum = 1;
		else
			iocb[handle].devnum = iocb[handle].dev_name[1] & 0x07;
	}
	return(i);
}

static find_free_iocb()
{
	register int i;

	for(i=0;i<32;++i)
	{
		if(iocb[i].ichid == -1)
			return(i);
	}
	return(-1);
}

static free_iocb(int handle)
{
	iocb[handle].ichid = -1;
	return(0);
}

void init_iocb()
{
	register int i;

	for(i=0;i<32;++i)
	{
		iocb[i].ichid = -1;
	}
	last_dev = 0;
}

