#ifndef CIO
#define CIO

typedef struct {
	union {
		struct {
			int (*openv)();	/*	open vector	*/
			int (*closv)();	/*	close vector	*/
			int (*getv)();	/*	get byte vector	*/
			int (*putv)();	/*	put byte vector	*/
			int (*statv)();	/*	get status vector	*/
			int (*specv)();	/*	special vector	*/
			int (*initv)();	/*	init code vector	*/
		}jt;
		int (*jumptab[7])();
	}jtab;
}H_JVEC;


/*
	offsets into jump table
*/

#define VOPEN	0
#define VCLOSE	1
#define VGET	2
#define VPUT	3
#define VSTAT	4
#define VSPECIAL	5

typedef struct {
	int dev_name;
	int future_exp;
	H_JVEC *vectors;	/*	pointer to handler jump table	*/
}HTABS;

typedef struct {
	int ichid;		/*	handler ID	*/
	int command;
	int devnum;		/*	device number	*/
	int status;
	char *dev_name;	/*	pointer to file name	*/
	char *buffer;	/*	pointer to buffer	*/
	long count;
	int mode;
	int aux;
	long exp1;	/*	future expansion	*/
	long exp2;
}IOCB;

/*
	function numbers
*/

#define	ADD_HANDLER	1	/*	add handler to device table	*/
#define	OPEN	3
#define	GETR	5
#define	GETC	7
#define	PUTR	9
#define	PUTC	11
#define	CLOSE	12
#define	STATUS	13

#define	MAX_HANDLE	19
#define	EOL	0x0a

/*
	cio error codes
*/
#define	ABORT	-1
#define	NO_HANDLES	-2	/*	NO MORE IOCB'S AVAILIABLE	*/
#define	NO_DEVICE	-3
#define	WRONLY	-4
#define	INVALID	-5	/*	INVALID FUNCTION NUMBER	*/
#define	NOT_OPEN	-6	/*	IOCB NOT OPEN	*/
#define	INVLD_HNDL	-7	/*	INVALID HANDLE	*/
#define	RDONLY	-8
#define	END_OF_FILE	-9
#define	TRUNCATED	-10	/*	NO EOL FOUND BEFORE END OF BUFFER	*/

/*
** Bindings to CIO
*/

/* Add_handler(n,t) n: device name, t: pointer to jump table	*/

#define Add_handler(n,t)	(int)cio(ADD_HANDLER,(n),(t))
#define Open(n,m)		(int)cio(OPEN,(n),(m))
#define Close(h)		(int)cio(CLOSE,(h))
#define	Getc(x)			(int)cio(GETC,(x),0l)
#define Putc(x,a)		(int)cio(PUTC,(x),0l,(a))
#define	Status(h,b,c,a,d)	(int)cio(STATUS,(h),(b),(c),(a),(d))

/*
** Xio:
** c: COMMAND
** h: Handle
** b: buffer
** n: byte count
** a: aux
** d: device name (pointer to string)
*/

#define Xio(c,h,b,n,a,d)	cio((c),(h),(b),(n),(a),(d))

/*
** file modes
*/
#define READ_ONLY	4
#define WRITE_ONLY	8

void init_iocb(void);
int cio(int,...);

#endif
