/*
**  Debug consol for Furnace Controller
*/
#include <stdio.h>
#include <stdlib.h>
#include <curses.h>
#include <ctype.h>
#include <string.h>
#include <cio.h>
#include "timer.h"
#include "midi.h"
#include "rs232.h"

extern long strtol();
static void display_midi(WINDOW*);
extern void *dis(void*,char*);
void *get_address(void);
static void write_contin(WINDOW *w);
static void read_contin(WINDOW *w);
static void mem_dump(WINDOW *win);
static void display_midi(WINDOW *win);
static void DoMidi(WINDOW *);
#define P_PORT 0xd0000
/*
** Menu Strings
*/

static const char *menu_str[] = {
	"MAIN MENU",
	"A. MEM DUMP",
	"B. POKE LOC",
	"C. Display Midi",
	"D. Test Out Lights",
	"E. DISASMEBLE",
	"F. Do Midi Thing",
	"G. READ LOC CONT",
	"H. WRITE LOC CONT",
	"I. Debug Timer",
	"X. EXIT",
	NULL
};


#ifdef MAIN

main()
{
	consol();
}

#endif

consol()	/*	main consol routine	*/
{
	WINDOW *main_menu,*action;
	int i,a;
	int *m;

	initscr();	/*	init curses	*/
	boxed();
	crmode();
	noecho();
	main_menu = newwin(20,30,1,1);
	for(i=0;menu_str[i] != NULL;++i)
		wprintw(main_menu,"%s\n",menu_str[i]);
	action = newwin(23,70,1,5);
	scrollok(action,1);
	wprintw(action,"Do Midi\n");
	DoMidi(action);
	delwin(action);
	while(1)
	{
		a = wgetch(main_menu);
		action = newwin(23,70,1,5);
		switch(toupper(a))
		{
			case 'A':
				scrollok(action,1);
				wprintw(action,"MEM DUMP\n");
				mem_dump(action);
				break;
			case 'B':	/*	poke	*/
				wprintw(action,"POKE MEMORY\n");
				do_poke(action);
				break;
			case 'C':
				scrollok(action,1);
				wprintw(action,"DISPLAY MIDI DATA\n");
				display_midi(action);
				break;
			case 'D':
				wprintw(action,"Testing Output Relay Lights\n");
				test_lites(action);
				break;
			case 'E':
				scrollok(action,1);
				wprintw(action,"Disasemble memory\n");
				disasemble_mem(action);
				break;
			case 'F':
				scrollok(action,1);
				wprintw(action,"Do Midi\n");
				DoMidi(action);
				break;
			case 'G':
				wprintw(action,"READ A LOCATION CONTINUOUSLY\n");
				read_contin(action);
				break;
			case 'H':
				wprintw(action,"WRITE A LOCATION CONTINUOUSLY\n");
				write_contin(action);
				break;
			case 'I':
				{
					int t = 0;

					do
					{

						wprintw(action,"Timer val = %d\n",ticks);
						if(Status(-1,(void *)0,0l,RECEIVE_STAT,"R:") > 0)
							t = wgetch(action);		/*	get a character from the keyboard	*/
					}while(toupper(t) != 'X');		/*	do forever for now	*/
				}
				break;
			default:
				wprintw(action,"%x %c\n",a,a);
				break;
		}	/*	end of switch statement	*/
		delwin(action);
	}	/*	end of while loop	*/
}

disasemble_mem(win)
WINDOW *win;
{
	int *m;
	int i;
	char s[80];

/*	return;          */
	m = (int *)get_address();
	wprintw(win,"Start Address = %lx\n",(long)m);
	do
	{
		for(i=0;i<12;++i)
		{
			if((m = (int *)dis((void *)m,s)) == NULL)
			{
				wprintw(win,"Disasembly ERROR\n");
				wprintw(win,"Press any key to continue\n");
				i = wgetch(win);
				return;
			}
			else
				wprintw(win,"%s\n",s);
		}
		wprintw(win,"X to exit,other to continue\n");
		i = wgetch(win);
	} while (toupper(i) != 'X');
}

static void mem_dump(WINDOW *win)
{
	int i;
	int j;
	int a;
	long adr;
	char *p;
	int *ip;
	char *sp;		/* place to put stuff that is printing	*/
	char *ap;

	if((sp = malloc(100)) == NULL)
	{
		wprintw(win,"Could not allocate memory\n");
		return;
	}
	adr = (long)get_address();
	p = (char *)adr;
	ip = (int *)adr;
	do
	{
		for(i=0;i<16;++i)
		{
			ap = sp;
			ap = ap + sprintf(ap,"%6lx ",adr);
			for(j=0;j<8;++j)
			{
				ap = ap + sprintf(ap,"%4x ",ip[j]);
			}
			for(j=0;j<16;++j)
			{
				a = (int)p[j] & 0xff;
				if(a < ' ' || a > 127)
					a = '.';
/*				waddch(win,a);		*/
				*ap = a;
				++ap;
			}
			adr += 16;
			p = (char *)adr;
			ip = (int *)adr;
			*ap = '\0';
			wprintw(win,"%s\n",sp);
		}	/*	end for i loop	*/
		wprintw(win,"Any key to continue, X to EXIT\n");
		a = wgetch(win);
	}while( (toupper(a)) != 'X');
	free(sp);
}

get_string(s,p,l)
char *s;	/*	place to put string to get	*/
char *p;	/*	prompt	*/
int l;		/*	lenth of string to get	*/
{
	WINDOW *dialog;	/*	make a dialog box	*/
	int w;			/*	calculate a width	*/

	w = strlen(p) + l;	/*	how wid to make dialog box	*/
	dialog = newwin(3,w + 2,10,(80 - (w + 2)) >> 1);
	wprintw(dialog,p);	/*	print prompt	*/
	wgets(dialog,s);
	delwin(dialog);
}

void *get_address()
{
	char t[10],*p;

	get_string(t,"Enter HEX Address:",6);
	return((void *)strtol(t,&p,16));
}

do_poke(w)
WINDOW *w;
{
	char *address;
	int data;
	char d[80];
	char *a;

	get_string(d,"ADDRESS:",8);
	address = (char *)strtol(d,&a,16);
	get_string(d,"DATA:",4);
	data = (int)strtol(d,&a,16);
	*address = (char)data;
}

static void read_contin(WINDOW *w)
{
	void *adr;
	int a;
	int size;
	int val;
	int t;

	adr = get_address();		/*	get the address to read	*/
loop:
	wprintw(w,"Word or Byte access? (W or B):");
	a = wgetch(w) & 0x7f;				/*	get response	*/
	switch(toupper(a))
	{
		case 'W':
			size = 1;
			break;
		case 'B':
			size = 0;
			break;
		default:
			wprintw(w,"Incorrect Entry, Try again\n");
			goto loop;
			break;
	}
	do
	{
		if(size)
			val = *((int *)adr);
		else
			val = (int)*((char *)adr);
		if(Status(-1,(void *)0,0l,RECEIVE_STAT,"R:") > 0)
			t = wgetch(w);		/*	get a character from the keyboard	*/
	}while(toupper(t) != 'X');		/*	do forever for now	*/
}

static void write_contin(WINDOW *w)
{
	void *adr;
	int a,t;
	int size,user;
	int val;
	int i,del;
	char *s,d[80];
	int delay;

	adr = get_address();		/*	get the address to read	*/
loop:
	wprintw(w,"Word or Byte access? (W or B):");
	a = wgetch(w) & 0x7f;				/*	get response	*/
	wprintw(w,"%c\n",a);
	switch(toupper(a))
	{
		case 'W':
			size = 1;
			break;
		case 'B':
			size = 0;
			break;
		default:
			wprintw(w,"Incorrect Entry, Try again\n");
			goto loop;
			break;
	}
loop1:
	wprintw(w,"User Value or Auto Inc (U or A)?:");
	a = wgetch(w) & 0x7f;
	wprintw(w,"%c\n",a);
	switch(toupper(a))
	{
		case 'A':
			user = 0;		/*	auto increment	*/
			break;
		case 'U':
			user = 1;
			get_string(d,"DATA:",4);
			val = (int)strtol(d,&s,16);
			break;
		default:
			wprintw(w,"Incorrect Entry, Try again\n");
			goto loop1;
			break;
	}
loop2:
	wprintw(w,"Delay:");
	get_string(d,"DATA:",4);
	delay = (int)strtol(d,&s,16);
	wprintw(w,"%x\n",delay);
	do
	{
		if(size)
		{
			if(user)
				*((int *)adr) = val;
			else
				*((int *)adr) = i++;
		}
		else
		{
			if(user)
				*((char *)adr) = (char)val;
			else
				*((char *)adr) = (char)i++;
		}
		if(delay)
			for(del=delay;del > 0;--del);
		if(Status(-1,(void *)0,0l,RECEIVE_STAT,"R:") > 0)
			t = wgetch(w);		/*	get a character from the keyboard	*/
	}while(toupper(t) != 'X');		/*	do forever for now	*/
}

test_lites(WINDOW *w)
{
	unsigned char lites[8];
	int i;
	int state = 0;
	int direction = 1;

	for(i=0;i<8;++i)
		lites[i] = 0;
	lites[0] = 0x1;

	do
	{
		for(i=0;i<8;++i)
			*((unsigned char *)(0xd0020 + i)) = ~lites[i];

		if(direction > 0)
			lites[state] <<= 1;
		else
			lites[state] >>= 1;

		if(lites[state] == 0 )
		{
			state += direction;
			if(direction > 0)
			{
				if(state > 7)
				{
					state = 7;
					direction = -1;
					lites[state] = 0x80;
				}
				else
				{
					lites[state] = 1;
				}
			}
			else
			{
				if(state < 0)
				{
					state = 0;
					direction = 1;
					lites[state] = 1;
				}
				else
				{
					lites[state] = 0x80;
				}
			}
		}
		ticks = 1;
		while(ticks > 0);
	}while(1);

}

static void display_midi(WINDOW *win)
{
	/*
	** This routine opens up the midi port, and displays midi data
	** on the screen in hex format
	*/
	int handle;	/*	handle for midi port	*/
	int i = 0;
	int a;
	int t;		/*	test rs232 status	*/

	wprintw(win,"*** Press X to exit ***\n");
	handle = Open("M:",READ_ONLY);
	do {
		;
		if (Status(-1,NULL,0l,RECEIVE_STAT,"M:") )
		{
			a = Getc(handle) & 0xff;	/*	get data	*/
			wprintw(win,"%2x ",a);
			++i;
			if(i>17)
			{
				i = 0;
				wprintw(win,"\n");
			}
		}
		if(Status(-1,(void *)0,0l,RECEIVE_STAT,"R:") > 0)
			t = wgetch(win);		/*	get a character from the keyboard	*/
	}while(toupper(t) != 'X');		/*	do forever for now	*/
	Close(handle);
}

static void DoMidi(WINDOW *w)
{
	int handle;	/*	handle for midi port	*/
	int t;
	int event,note,vel,bit,port;
	int pshad[4];
	register int i;
	static const int bitmap[8] = { 0x80,0x40,0x20,0x10,0x8,0x4,0x2,0x1};


	for(t=0;t<4;++t)
		pshad[t] = 0;
	handle = Open("M:",READ_ONLY);
	do {
		if (Status(-1,NULL,0l,RECEIVE_STAT,"M:") )
		{
			event = Getc(handle) & 0xff;	/*	get data	*/
			switch(event & 0xf0)
			{
				case 0x80:	/*	note off	*/
					note = (Getc(handle) & 0xff);	/*	get note	*/
					vel = Getc(handle) & 0xff;  /*	get velocity	*/
					note -= 31;
					if(note < 31 && note >= 0)
					{
						port = (note >> 3) & 3;	/* divide by 8	*/
						bit = note & 0x07;	/*	mod by 8	*/
						pshad[port] &= ~bitmap[bit];	/*	map bit	*/
						*((char *)(P_PORT + port)) = pshad[port];
					}
					break;
				case 0x90:  /*	note on	*/
					note = (Getc(handle) & 0xff);	/*	get note	*/
					vel = Getc(handle) & 0xff;  /*	get velocity	*/
/*					t = setup_timer(50,note);	*/		/*	timer for 100ms	*/
/*					wprintw(w,"%2x %2x %2x %d\n",event,note,vel,t);	*/
					note -= 31;
					if(note < 31 && note >= 0)
					{
						port = (note >> 3) & 3;	/* divide by 8	*/
						bit = note & 0x07;	/*	mod by 8	*/
						if(vel == 0)
							pshad[port] &= ~bitmap[bit];	/*	map bit	*/
						else
							pshad[port] |= bitmap[bit];	/*	map bit	*/
						*((char *)(P_PORT + port)) = pshad[port];
					}
					break;
				case 0xa0:
					break;
				case 0xb0:
					break;
				case 0xc0:
					break;
				case 0xd0:
					break;
				case 0xe0:
					break;
				case 0xf0:
					break;
			}	/*	end of switch event	*/
			for(i=0;i<4;++i)
				*((char *)(P_PORT + i)) = pshad[i];
		}	/*	end of check for midi event	*/
		if(Status(-1,(void *)0,0l,RECEIVE_STAT,"R:") > 0)
			t = wgetch(w);		/*	get a character from the keyboard	*/
	}while(toupper(t) != 'X');		/*	do forever for now	*/
	Close(handle);
}
