/*************************************************************
**                                                          **
** Header file for MC146818 Real Time Chip                  **
**                                                          **
*************************************************************/


/*
** Time Of Day structure
*/

typedef struct {
	union {
		struct {
			unsigned char seconds;
			unsigned char sec_alarm;
			unsigned char minutes;
			unsigned char min_alarm;
			unsigned char hours;
			unsigned char hour_alarm;
			unsigned char day_of_week;
			unsigned char date;
			unsigned char month;
			unsigned char year;
			unsigned char reg_a;
			unsigned char reg_b;
			unsigned char reg_c;
			unsigned char reg_d;
		}hi_lev;
		unsigned char lo_lev[14];
	}td;
}TOD;

#define TOD_BASE	0xa0000

/****************************
**                         **
** REG A defines           **
**                         **
****************************/

#define UIP			0x80	/*	update in progress status flag	*/
#define DIV_MASK	0x70	/*	used to enable the clock	*/
#define START_COUNT	0x20	/*	start osc and countdown		*/
#define START_HOLD	0x60	/*	start osc and hold count	*/
#define RS_MASK		0x0f	/*	select tap on 15 stage div	*/
#define REG_A_OFF	20
/****************************
**                         **
** REG B defines           **
**                         **
****************************/

#define SET			0x80	/*	disable the time update		*/
#define PIE			0x40	/*	periodic interrupt enable	*/
#define AIE			0x20	/*	alarm interrupt enable		*/
#define	UIE			0x10    /*	Update Ended Interrupt Enable	*/
#define	SQWE		0x08    /*	Square Wave enable			*/
#define	DM			0x04    /*	Data Mode BCD/BINARY		*/
#define DAY24		0x02    /*	12/24 hour day				*/
#define DSE			0x01    /*	Dalight Savings Time		*/
#define REG_B_OFF	22

/****************************
**                         **
** REG C defines           **
**                         **
****************************/

#define IRQF		0x80	/*	Interrupt request flag		*/
#define	PF			0x40	/*	Periodic Interrupt flag		*/
#define	AF			0x20	/*	Alarm Flag					*/
#define	UF			0x10	/*	Update Ended Flag			*/
#define REG_C_OFF	24

/****************************
**                         **
** REG D defines           **
**                         **
****************************/

#define VRT			0x80	/*	valid Time bit for Dallas chip	*/
#define REG_D_OFF	26


/***************************
**                        **
**	Special I/O functions **
**                        **
***************************/

#define GET_TOD_BLOCK	0x20
#define UPDATE_TOD		0x21
#define SET_ALARM_SEC	0x22
#define SET_ALARM_MIN	0x23
#define SET_ALARM_HR	0x24

/***************************
**
** modes for OPEN
**
***************************/

#define TIME_READ		0x04		/*	binary	*/
#define WRITE_TIME		0x08
#define TIME_RD_ASC		0x84		/*	ascii	*/
#define TIME_WD_ASC		0x88
#define ASCII_MODE		0x80

/***************************
**
** time format defines
**
***************************/

#define SUNDAY	1
#define MONDAY	2
#define TUESDAY	3
#define WEDNESDAY	4
#define THURSDAY	5
#define FRIDAY	6
#define SATURDAY	7

#define JANUARY	1
#define FEBUARY	2
#define MARCH	3
#define APRIL	4
#define	MAY		5
#define	JUNE	6
#define JULY	7
#define AUGUST	8
#define SEPTEMBER	9
#define OCTOBER	10
#define NOVEMBER	11
#define DECEMBER	12

/***************************
**
** MISC
**
***************************/

#define MAX_ASCII		20		/*	maximum characters in time string	*/


extern int T_init(void);	/*	only public access to time device driver	*/
