;	module name rs232
;       assembly code for RS232 port
;
;	external declarations
;
	XDEF	_M_put,_init_midi_buff,_M_get
	XREF	_imr_shad
;
;

UART: 			EQU 0xfffc0000
STATUS_REG_B: 		EQU	9
MODE_REG_B: 		EQU	8
COMMAND_REG_B: 		EQU	10
DATA_REG_B: 		EQU	11
AUX_CONTROL_REG: 	EQU	4
INPUT_PORT_CHANGE_REG: 	EQU	4
INTERRUPT_STATUS_REG: 	EQU	5
INTERRUPT_MASK_REG: 	EQU	5
COUNTER_UPPER_REG: 	EQU	6
COUNTER_LOWER_REG: 	EQU	7
INTERRUPT_VECTOR_REG: 	EQU	12
INPUT_PORT_REG: 	EQU 	13
OUTPUT_PORT_CONFIG: 	EQU	13
START_COUNTER: 		EQU	14
BIT_SET_COMMAND: 	EQU	14
STOP_COUNTER: 		EQU	15
BIT_RESET_COMMAND: 	EQU	15
OUTPUT_PORT: 		EQU	14



BUFFER: = 0
HEAD:	= 4
TAIL:	= 6
SIZE:	= 8
NCHARS:	   = 10
HIGH:	= 12		;high water marker
LOW:	= 14		;low water marker

	SECTION data

save: DC.W	0	;

	SECTION ram	;declare data regions for rs232 buffers
	XDEF _midi_r_rec,_midi_t_rec

_midi_t_rec:	DS.W 6			;transmit output descriptor
_midi_r_rec:	DS.W  6			;recieve input descriptor
midi_t_buff:	DS.B 256		;transmit output buffer
midi_r_buff:	DS.B 256		; recieve input buffer

	SECTION code


_init_midi_buff:
	movea.l #_midi_t_rec,a0
	move.l	#midi_t_buff,BUFFER(a0)	;set up transmit buffer pointer
	move.w	#0,HEAD(a0)				;set up head pointer
	move.w	#0,TAIL(a0)				;set up tail pointer
	move.w	#256,SIZE(a0)			;set up buffer size
	move.w	#0,NCHARS(a0)			;init number of characters counter
;	move.b	#0,_imr_shad
	movea.l	#_midi_r_rec,a0
	move.l	#midi_r_buff,BUFFER(a0)	;set up buffer pointer
	move.w	#0,HEAD(a0)				;set up head pointer
	move.w	#0,TAIL(a0)				;set up tail pointer
	move.w	#256,SIZE(a0)			;set up recieve buffer size
	move.w	#0,NCHARS(a0)       	;init number of characters counter
	move.w	#225,HIGH(a0)			;set up high water mark
	move.w	#25,LOW(a0)				;set up low water mark
	move.b	_imr_shad,UART+INTERRUPT_MASK_REG
	rts

_M_put:

DATA: EQU 12

	link	a6,#0			;stack frame points to parameters
	move	sr,-(a7)		;Save the status register
	movea.l #_midi_t_rec,a0		;get pointer to output descriptor
	ori.w	#0x700,sr		;disable interrupts
	btst	#2,UART+STATUS_REG_B	;is uart ready to xmit?
	beq.s	put1			;no it ain't, use circ buffer
	move	HEAD(a0),d1
	cmp.w	TAIL(a0),d1		;any thing in circ buffer?
	bne.s	put1			;yes, so use circ buffer
	move	DATA(a6),d0		;no, put directly into UART
	move.b	d0,UART+DATA_REG_B
	bra	put2
put1:
	btst	#4,_imr_shad
	bne.s	put5
	bset	#4,_imr_shad	;enable transmit interrupts
	move.b	_imr_shad,UART+INTERRUPT_MASK_REG
put5:
	move	TAIL(a0),d1	;get tail pointer
;
;
;
	addq.w #1,d1		;increment tail pointer
	cmp.w	SIZE(a0),d1	;does it need to WRAP?
	bcs	wo1		;No, it is just fine
	moveq	#0,d1		;set tail back to beginning of buffer
wo1:
	cmp.w	HEAD(a0),d1	;HEAD == TAIL?
;	beq.s	put2		;just leave (wrong)
	bne.s	put3		;No, put data in buffer
				;Yes, need to wait for space in
				;output buffer
	move	(a7)+,sr	;pop the status register to restore IRQ
put4:
	cmp.w	HEAD(a0),d1	;wait for room in buffer
	beq	put4
	move	sr,-(a7)	;Save the status register
	ori.w	#0x700,sr	;disable interrupts
	bra	wo1		;try this again

put3:
	movea.l	BUFFER(a0),a1	;get the address of the buffer
	move	DATA(a6),d0	;get data from off the stack
	move.b	d0,0(a1,d1)	;put data into circular buffer
	move	d1,TAIL(a0)	;update tail pointer
put2:
	move	(a7)+,sr	;restore status register
	unlk	a6		;deallocate stack space
	rts			;get out of here


_M_get:
	link	a6,#0		;allocate space on stack
rg_loop:
	movea.l	#_midi_r_rec,a0	;pointer to buffer descriptor
	move.w	NCHARS(a0),d0   ;load number of characters in buffer
	beq.s	rg_loop			;wait until some data is in the buffer
	;
	;
	;main part of rs232 interface
	;
get21:
	move.w	sr,-(a7)	;save status reg
	ori		#0x0700,sr	;disable interrupts
	move	HEAD(a0),d1	;get head pointer
	addq.w	#1,d1		;increment head pointer
	cmp.w	SIZE(a0),d1	;wrap around?
	bcs.s	get1		;no
	moveq	#0,d1		;yes, do a WRAP
get1:
	movea.l	BUFFER(a0),a1	;get address of data buffer
	move.b	0(a1,d1),d0	;get data byte from buffer
	andi.w	#0x0ff,d0	;zap out upper part of register
	move.w	d1,HEAD(a0)	;save head pointer
	subq.w	#1,NCHARS(a0)	;decrement character conter


	move.w	NCHARS(a0),d1	;if recieve interrupt is disable,
	cmp.w	LOW(a0),d1      ;check low water mark
	bgt.s	get5			;branch if not below and exit
	move.b	#2,UART+BIT_SET_COMMAND	;set RTS line

;	bset.b	#5,_imr_shad	;re-enable interrupt for reciever
;	move.b	_imr_shad,UART+INTERRUPT_MASK_REG

;	move.w	TAIL(a0),d1		;index into rs232 ring buffer
;	addq	#1,d1			;increment tail pointer
;	cmp.w	SIZE(a0),d1		;i >EQU to current size?
;	bcs.s	rmid_p1			;buffer wrap?
;	moveq	#0,d1			;shore did, do wrap
rmid_p1:
;	btst.b	#0,UART+STATUS_REG_B	;is anything there?
;	beq.s	get5
;	move.w	d0,-(a7)
;	move.b	UART+DATA_REG_B,d0	;get data from port and clear interrupt

;	move.b	d0,0(a1,d1)		;save data into buffer
;	move.w	(a7)+,d0
;	move.w	d1,TAIL(a0)
;	addq.w	#1,NCHARS(a0)	;increment number of characters in buffer

get5:
	move	(a7)+,sr		;restore status register
	unlk	a6				;release space on stack
	rts

