/*
	MIDI handler for poly synth
*/
#include "midi.h"
#include <cio.h>
#define TRUE 1
#define FALSE 0

int M_open();
int M_close();
extern int M_get();
extern int M_put();
int M_status();
int M_xio();
int M_init(void);

static const H_JVEC m_vec = {	/*	handler table	*/
	M_open,
	M_close,
	M_get,
	M_put,
	M_status,
	M_xio,
	M_init
};

extern char imr_shad;
extern int disable(),enable();
/*
** initialize the MIDI handler
*/
static init_uart(void);	/*	initialize MIDI uart	*/

int M_init()
{
	
	init_uart();
	init_midi_buff();
	Add_handler('M',&m_vec);
}

static init_uart()
{
	*((char *)(UART + COMMAND_REG_B)) = 0x10;	/* reset MR ptr */
	*((char *)(UART + BIT_SET_COMMAND)) = 0x02;	/*	assert RTS	*/
	*((char *)(UART + MODE_REG_B)) = 0x93;		/*	no parity, 8 bits	*/
	*((char *)(UART + MODE_REG_B)) = 0x07;		/*	1 stop bit	*/
	*((char *)(UART + COMMAND_REG_B)) = 0x20;   /* reset rxr	*/
	*((char *)(UART + COMMAND_REG_B)) = 0x30;   /* reset TXD */
	*((char *)(UART + STATUS_REG_B) ) = 0xee; 	/* IP2/5 16x clock	*/
	*((char *)(UART + COMMAND_REG_B)) = 0x04; 	/*	transmit enabled	*/
	*((char *)(UART + COMMAND_REG_B)) = 0x01;   /* recieve enabled */

}

M_open(iocb)
IOCB *iocb;
{
	if (iocb->mode & READ_ONLY)
	{
		/*
		**	enable system interrupts
		*/
		disable();	/*	disable interrupts	*/
		imr_shad |= RX_B_RDY_IRQ;	/*	enable interrupt bit	*/
		*((char *)(UART + INTERRUPT_MASK_REG)) = imr_shad;
		enable();
	}
	return(0);
}

M_close(iocb)
IOCB *iocb;
{
	/*
	** disable the UART interrupts
	*/
	disable();
	imr_shad &= ~(RX_B_RDY_IRQ | TX_B_RDY_IRQ | DELTA_BREAK_B);
	*((char *)(UART + INTERRUPT_MASK_REG)) = imr_shad;
	enable();
	return(0);
}

M_xio(iocb)
IOCB *iocb;
{
	extern IO_REC midi_t_rec,midi_r_rec;
	/*
	** execute misc function needed by MIDI port
	*/
	switch(iocb->command)	/*	decode command	*/
	{
		case SET_BAUD_RATE:
			/*
			** baud rate is in aux (an int)
			*/
			*((char *)(UART + STATUS_REG_B)) = (char)iocb->aux;
			break;
		case SEND_BREAK:
			break;
		case WAIT_EMPTY:
			/*	wait for transmit buffer to empty	*/
			while(midi_t_rec.tail != midi_t_rec.head);	/*	wait for index to be equal	*/
			break;
		case SET_MODE:
			break;
		case SET_INPUT_BUFFER:
			break;
		case SET_OUTPUT_BUFFER:
			break;
	}	/*	end of switch command	*/
	return(0);
}

M_status(iocb)
IOCB *iocb;
{
	extern IO_REC midi_t_rec,midi_r_rec;

	switch(iocb->aux)		/*	which status function	*/
	{
		case RECEIVE_STAT:
			return midi_r_rec.nchars;	/*	return number of characters	*/
		case SEND_STAT:
			return midi_t_rec.nchars;	/*	return number of xmit chars	*/
	}


	return(midi_r_rec.nchars);		/*	default, return recieve chars	*/

}
