;	module name rs232
;       assembly code for RS232 port
;
;	external declarations
;
	XDEF	_R_put,_init_buff,_R_get,_disable
	XDEF	_enable
	XREF	_imr_shad
;
;


UART: 			EQU 0xfffc0000
STATUS_REG_A: 		EQU	1
MODE_REG_A: 		EQU	0
COMMAND_REG_A: 		EQU	2
DATA_REG_A: 		EQU	3
AUX_CONTROL_REG: 	EQU	4
INPUT_PORT_CHANGE_REG: 	EQU	4
INTERRUPT_STATUS_REG: 	EQU	5
INTERRUPT_MASK_REG: 	EQU	5
COUNTER_UPPER_REG: 	EQU	6
COUNTER_LOWER_REG: 	EQU	7
INTERRUPT_VECTOR_REG: 	EQU	12
INPUT_PORT_REG: 	EQU 	13
OUTPUT_PORT_CONFIG: 	EQU	13
START_COUNTER: 		EQU	14
BIT_SET_COMMAND: 	EQU	14
STOP_COUNTER: 		EQU	15
BIT_RESET_COMMAND: 	EQU	15
OUTPUT_PORT: 		EQU	14


BUFFER: = 0
HEAD:	= 4
TAIL:	= 6
SIZE:	= 8
NCHARS:	= 10
HIGH:	= 12
LOW:	= 14

	SECTION data
	XDEF	SR_save

SR_save: DC.W	0	;

	SECTION ram	;declare data regions for rs232 buffers
	XDEF _r_rec,_t_rec

_r_rec:	DS.W  8			;recieve input descriptor
_t_rec:	DS.W  8			;transmit output descriptor
r_buff:	DS.B 256		; recieve input buffer
t_buff:	DS.B 256		;transmit output buffer

	SECTION code


_init_buff:
	movea.l #_t_rec,a0
	move.l	#t_buff,BUFFER(a0)	;
	move.w	#0,HEAD(a0)			;
	move.w	#0,TAIL(a0)			;
	move.w	#256,SIZE(a0)			;
	move.w	#0,NCHARS(a0)			;
	move.b	#0,_imr_shad
	movea.l	#_r_rec,a0		;
	move.l	#r_buff,BUFFER(a0)	;
	move.w	#0,HEAD(a0)		;
	move.w	#0,TAIL(a0)		;
	move.w	#256,SIZE(a0)		;
	move.w	#0,NCHARS(a0)
	move.w	#25,LOW(a0)			;init low water mark
	move.w	#225,HIGH(a0)		;init high water mark
	move.b	_imr_shad,UART+INTERRUPT_MASK_REG
	rts

_R_put:

DATA: EQU 12

	link	a6,#0			;stack frame points to parameters
	move	sr,-(a7)		;Save the status register
	movea.l #_t_rec,a0		;get pointer to output descriptor
	ori.w	#0x700,sr		;disable interrupts
	btst	#2,UART+STATUS_REG_A	;is uart ready to xmit?
	beq.s	put1			;no it ain't, use circ buffer
	move.w	NCHARS(a0),d1   ;load number of characters in buffer
	bne.s	put1			;yes, so use circ buffer
	move.w	DATA(a6),d0		;no, put directly into UART
	move.b	d0,UART+DATA_REG_A
	bra	put2				;exit

put1:
	btst.b	#0,_imr_shad
	bne.s	put5
	bset.b	#0,_imr_shad	;enable transmit interrupts
	move.b	_imr_shad,UART+INTERRUPT_MASK_REG
put5:
	move.w 	NCHARS(a0),d1	;get number of characters in buffer
	cmp.w	SIZE(a0),d1		;is the buffer full?
	blt.s	wo2             ;
	bsr.s	put_wait		;wait for space to be availiable
wo2:
	move.w	TAIL(a0),d1		;get tail pointer
	addq.w #1,d1			;increment tail pointer
	cmp.w	SIZE(a0),d1		;does it need to WRAP?
	bcs	wo1					;No, it is just fine
	moveq	#0,d1			;set tail back to beginning of buffer

wo1:
	movea.l	BUFFER(a0),a1	;get the address of the buffer
	move.w	DATA(a6),d0		;get data from off the stack
	move.b	d0,0(a1,d1)		;put data into circular buffer
	move.w	d1,TAIL(a0)		;update tail pointer
	addq.w	#1,NCHARS(a0)	;increment number of characters in buffer
	bra.s	put2			;exit


put_wait:
	move.w	4(a7),sr		;get the status register to restore IRQ
	move.w	SIZE(a0),d1
put4:
	cmp.w	NCHARS(a0),d1	;wait for room in buffer
	beq		put4
	ori.w	#0x700,sr		;disable interrupts
	rts

put2:
	move	(a7)+,sr		;restore status register
	unlk	a6				;deallocate stack space
	rts						;get out of here


_R_get:
	link	a6,#0		;allocate space on stack
	movem.l	a2/a3,-(a7)		;	we use a2 and a3,so save them
rg_loop:
	bsr.s	com_stat	;check for data in input buffer
	tst	d0
	beq.s	rg_loop;	;no data, keep looping
	move.w	sr,-(a7)	;save status reg
	ori	#0x0700,sr	;disable interrupts
	move	HEAD(a0),d1	;get head pointer
	cmp.w	TAIL(a0),d1	;are they equal?
	beq.s	get1		;
;
;
;
	addq.w	#1,d1		;increment head pointer
	cmp.w	SIZE(a0),d1	;wrap around?
	bcs.s	get1		;no
	moveq	#0,d1		;yes, do a WRAP
get1:
	movea.l	BUFFER(a0),a1	;get address of data buffer
	move.b	0(a1,d1),d0	;get data byte from buffer
	andi.w	#0x0ff,d0	;zap out upper part of register
	move.w	d1,HEAD(a0)	;save head pointer
	subq.w	#1,NCHARS(a0);decrement number of characters in buffer
	move	(a7)+,sr	;restore status register
	movem.l	(a7)+,a2/a3	;restore a2 and a3
	unlk	a6		;release space on stack
	rts

;
;
;

com_stat:
	movea.l	#_r_rec,a0	;get address of input descriptor
	moveq	#-1,d0		;set initial status
	lea	HEAD(a0),a2	;address of head pointer
	lea	TAIL(a0),a3	;address of tail pointer
	cmpm.w	(a3)+,(a2)+	;are they equal?
	bne.s	com_stat1	;no , just exit
	moveq	#0,d0		;yes, set d0 to zero
com_stat1:
	rts

;
;
;
;
;
;
;
_disable:
	move	sr,d0
	move	d0,SR_save
	ori	#0x0700,sr	;
	rts


_enable:
	move	SR_save,d0
	move	d0,sr	;
	rts
