/*
	test program for poly 5 synth

*/
#include <stdio.h>
#include "rs232.h"
#include <cio.h>
#include <curses.h>
#include <stdarg.h>

int R_open();
int R_close();
extern int R_get();
extern int R_put();
int R_status();
int R_xio();
int R_init(void);

static const H_JVEC r_vec = {
	R_open,
	R_close,
	R_get,
	R_put,
	R_status,
	R_xio,
	R_init
};

extern char imr_shad;
extern int disable(),enable();
extern int p_error(int,...);

/*
** global rs232 variables
*/
char acr_shad;

/*
** initialize the RS-232 handler
*/
static init_uart(void);

R_init()
{
	
	init_uart();
	init_irq();	/*	initialize interrupts for UART	*/
	init_buff();
	Add_handler('R',&r_vec);
}

static init_uart()
{
	*((char *)(UART + INTERRUPT_MASK_REG)) = 0x0;	/* DISABLE INTERRUPTS */
	*((char *)(UART + COMMAND_REG_A)) = 0x10;	/* reset MR ptr */
	*((char *)(UART + COMMAND_REG_A)) = 0x20;   /* reset rxr	*/
	*((char *)(UART + COMMAND_REG_A)) = 0x30;   /* reset TXD */
	*((char *)(UART + MODE_REG_A)) = 0x13;		/*	no parity, 8 bits	*/
	*((char *)(UART + MODE_REG_A)) = 0x07;		/*	1 stop bit	*/
	*((char *)(UART + STATUS_REG_A) ) = 0xbb;	/*	9600 buad	*/
	*((char *)(UART + COMMAND_REG_A)) = 0x04;	/*	transmit enabled	*/
	*((char *)(UART + COMMAND_REG_A)) = 0x01;   /* recieve enabled */
	acr_shad = 0x8c;
	*((char *)(UART + AUX_CONTROL_REG)) = acr_shad;	/*	enable change bits on ip2 and ip3	*/
}


R_open(iocb)
IOCB *iocb;
{
	if (iocb->mode & READ_ONLY)
	{
		/*
		**	enable system interrupts
		*/
		disable();	/*	disable interrupts	*/
		imr_shad |= RX_A_RDY_IRQ;	/*	enable interrupt bit	*/
		*((char *)(UART + INTERRUPT_MASK_REG)) = imr_shad;
		enable();
	}
	return(0);
}

R_close(iocb)
IOCB *iocb;
{
	/*
	** disable the UART interrupts
	*/
	disable();
	imr_shad &= ~(RX_A_RDY_IRQ | TX_A_RDY_IRQ | DELTA_BREAK_A);
	*((char *)(UART + INTERRUPT_MASK_REG)) = imr_shad;
	enable();
	return(0);
}

R_xio(iocb)
IOCB *iocb;
{
	extern IO_REC t_rec,r_rec;
	/*
	** execute misc function needed by RS-232 port
	*/
	switch(iocb->command)	/*	decode command	*/
	{
		case SET_BAUD_RATE:
			/*
			** baud rate is in aux (an int)
			*/
			*((char *)(UART + STATUS_REG_A)) = (char)iocb->aux;
			break;
		case SEND_BREAK:
			break;
		case WAIT_EMPTY:
			/*	wait for transmit buffer to empty	*/
			while(t_rec.tail != t_rec.head);	/*	wait for index to be equal	*/
			break;
		case SET_MODE:
			break;
		case SET_INPUT_BUFFER:
			break;
		case SET_OUTPUT_BUFFER:
			break;
		case MEM_PROTECT:	/*	set memprotect hardware line	*/
			/*
			** Aux = 0->mem protect off
			** Aux = 1->mem protect on
			*/
			if(iocb->aux)
				*((char *)(UART+BIT_RESET_COMMAND)) = SET_PROTECT_BIT;
			else
				*((char *)(UART+BIT_SET_COMMAND)) = SET_PROTECT_BIT;
			break;
		case SET_ERROR:
			/*
			** Aux = 0->turn error light off
			** Aux = 1->turn error light on
			*/
			if(iocb->aux)
				*((char *)(UART+BIT_RESET_COMMAND)) = SET_ERROR_BIT;
			else
				*((char *)(UART+BIT_SET_COMMAND)) = SET_ERROR_BIT;
			break;
		case SET_ACTIVE:
			/*
			** Aux = 0->turn active light off
			** aux = 1->turn active light on
			*/
			if(iocb->aux)
				*((char *)(UART+BIT_RESET_COMMAND)) = SET_ACTIVE_BIT;
			else
				*((char *)(UART+BIT_SET_COMMAND)) = SET_ACTIVE_BIT;
			break;
	}	/*	end of switch command	*/
	return(0);
}

R_status(iocb)
IOCB *iocb;
{
	/*
	** This code returns the number of bytes in the
	** recieve buffer
	*/
	extern IO_REC r_rec;    /*	receive descriptor	*/
	extern IO_REC t_rec;	/*	transmit descriptor	*/

	switch(iocb->aux)		/*	which status function	*/
	{
		case RECEIVE_STAT:
			return r_rec.nchars;	/*	return number of characters	*/
		case SEND_STAT:
			return t_rec.nchars;	/*	return number of xmit chars	*/
	}


	return(r_rec.nchars);		/*	default, return recieve chars	*/
}

p_error(int except,...)
{
	va_list args;
	WINDOW *bomb;

	va_start(args,except);
	bomb = newwin(9,50,7,15);	/*	window for bomb info	*/
	switch(except)
	{
		case 2:
			Xio(SET_ERROR,-1,NULL,0l,1,"R:");
			wprintw(bomb,"BUS ERROR\n");
print_bomb:		wprintw(bomb,"FUNCTION:%x\n",va_arg(args,int));
			wprintw(bomb,"ACCESS ADDRESS %lx\n",va_arg(args,long) );
			wprintw(bomb,"Instruction:%x\n",va_arg(args,int) ); 
regular:		wprintw(bomb,"STATUS:%x\n",va_arg(args,int) );
			wprintw(bomb,"PROGRAM COUNTER:%lx\n",va_arg(args,long) );
			break;
		case 3:
			Xio(SET_ACTIVE,-1,NULL,0l,1,"R:");
			wprintw(bomb,"ADDRESS ERROR\n");
			goto print_bomb;
			break;
		default:
			wprintw(bomb,"Unknown\n");
			goto regular;
			break;
	}
	while(1);	/*	loop here forever	*/
}
