; FEB 17,1991
;fixed transmit routine to send right character from transmit buffer
;
;feb 23, 1991
; added MIDI serial handler to code
;
;	module name rs232irq
	.OPTION	target=68000,flags=gG
	.OPTION	char=1,short=2,int=2,long=4,float=4,double=8
	.OPTION	ldouble=8,ptrd=4,ptrf=4,sizeof=4,cver=0x430
;	equates
	XDEF	_irq_vecs
	XDEF	_init_irq
	XDEF _rs232_irq
	XDEF _rx_midi
	XDEF _tx_midi
	XDEF _rx_port
	XDEF _tx_port
	XDEF _break_b
	XDEF _break_a
	XDEF timer
	XDEF _in_port_c
	XDEF	_imr_shad




MR1A:	EQU	0xfffc0000	/*	mode control register A	*/
MR2A:	EQU	0xfffc0000	/*	same address, different reg	*/
SRA:	EQU	0xfffc0001	/*	status register port A	*/
RBA:	EQU	0xfffc0003	/*	recieve buffer A	*/
IPCR:	EQU	0xfffc0004	/*	input change register	*/
ISR:	EQU	0xfffc0005	/*	interrupt status register	*/
CUR:	EQU	0xfffc0006	/*	counter mode		*/
CLR:	EQU	0xfffc0007	/*	counter mode		*/
MR1B:	EQU	0xfffc0008	/* mode control register B	*/
MR2B:	EQU	0xfffc0008	/* same address, different reg	*/
SRB:	EQU	0xfffc0009	/*	status register port B	*/
RBB:	EQU	0xfffc000b	/*	recieve buffer B	*/
IVR:	EQU	0xfffc000c	/*	interrupt vector reg	*/
IPUL:	EQU	0xfffc000d	/*	input port, unlatched	*/
START_COUNTER:	EQU	0xfffc000e
STOP_COUNTER:	EQU	0xfffc000f
BITSET:	=	0xfffc000e
BITRESET:	=	0xfffc000f

/* WRITE locations, where different from READ	*/

CSRA:	EQU	0xfffc0001	/*	clock select register A	*/
CRA:	EQU	0xfffc0002	/*	command register A	*/
TBA:	EQU	0xfffc0003	/*	transmit buffer A	*/
ACR:	EQU	0xfffc0004	/*	Auxiliary Control Register	*/
IMR:	EQU	0xfffc0005	/*	interrupt mask register	*/
CTUR:	EQU	0xfffc0006	/*	counter/timer upper reg	*/
CTLR:	EQU	0xfffc0007	/*	counter/timer lower reg	*/
CSRB:	EQU	0xfffc0009	/*	clock select register B	*/
CRB:	EQU	0xfffc000a	/*	command reg B	*/
TRB:	EQU	0xfffc000b	/*	transmit register B	*/
OPCR:	EQU	0xfffc000d	/*	output port config register	*/
OPRBS:	EQU	0xfffc000e	/*	output port bit set command	*/
OPRRS:	EQU	0xfffc000f	/*	output port bit reset command	*/

RS232_VEC:	EQU	0x00000100	/*	user vector 64	*/


EXCEPTION_VEC:	EQU	0x08
; equates for data structures

BUFFER:	EQU	0       ;pointer to buffer area
HEAD:	EQU	4       ;head pointer
TAIL:	EQU	6       ;tail pointer
SIZE:	EQU	8       ;size of buffer
NCHARS:	   = 10     ;number of characters in buffer
HIGH:	=	12		;high water mark
LOW:	=	14      ;low water mark

	SECTION	ram
_imr_shad:	DS.B	1
isr_temp:	DS.B	1
_irq_vecs:	DS.L	8	/*	storage for interrupt vectors	*/

	SECTION code
_init_irq:			;	initialize interrupt vectors
	move.l	#_rx_port,_irq_vecs	;rs232 recieve
	move.l	#_rx_midi,_irq_vecs+8	;MIDI recieve
	move.l	#_tx_port,_irq_vecs+4	;rs232 transmit
	move.l	#_tx_midi,_irq_vecs+12	;MIDI transmit
	move.l	#_break_a,_irq_vecs+16
	move.l	#_break_b,_irq_vecs+20
	move.l	#timer,_irq_vecs+24    ;timer interrupt
	move.l	#_in_port_c,_irq_vecs+28
	move.b	#0x40,IVR;
	andi	#0xf8ff,sr		;	enable interrupts
	rts

_rs232_irq:
	movem.l	d0/d1/a0/a1,-(a7) ;	save registers on the stack
	move.b	ISR,d0			;get status of interrupts
	and.b	_imr_shad,d0	;mask off unwanted status bits
	move.b	d0,isr_temp
	btst.b	#5,isr_temp			;* port B recive
	beq.s	L7			;highest priority
	movea.l	_irq_vecs+8,a0
	jsr	(a0)

L7:
	btst.b	#4,isr_temp
	beq.s	L6
	movea.l	_irq_vecs+12,a0	;	port b transmit
	jsr	(a0)

L6:
	btst.b	#0,isr_temp			;*	port a transmit
	beq.s	L5
	movea.l	_irq_vecs+4,a0
	jsr	(a0)

L5:
	btst.b	#1,isr_temp		; check interrupt status for UART port a
	beq.s	L8
	movea.l	_irq_vecs,a0	;port a recieve
	jsr	(a0)

L8:
	btst.b	#3,isr_temp		;check for a timer interrupt
	beq.s	L11
	movea.l	_irq_vecs+24,a0
	jsr	(a0)
;L9:
;	btst.b	#2,isr_temp
;	beq.s	L10
;	movea.l	_irq_vecs+16,a0
;	jsr	(a0)
;L10:
;	btst.b	#6,isr_temp
;	beq.s	L11
;	movea.l	_irq_vecs+20,a0
;	jsr	(a0)
L11:
	btst.b	#7,isr_temp
	beq.s	L4
	movea.l	_irq_vecs+28,a0
	jsr	(a0)
L4:
	movem.l	(a7)+,d0/d1/a0/a1		;	pop registers on the stack
	rte


BUFF: EQU 8	;	offset on stack of buffer pointer
PORT: EQU 12	;	offset of port number on the stack

;	This recieve routine takes a pointer to the rs232 descriptor
;	and the port number to do its input
	XREF	_r_rec

_rx_port:
	movea.l	#_r_rec,a0		;input port descriptor
	movea.l	BUFFER(a0),a1	;pointer to rs232 ring buffer
	move	TAIL(a0),d1		;index into rs232 ring buffer
	addq	#1,d1			;increment tail pointer
	cmp.w	SIZE(a0),d1		;i >EQU to current size?
	bcs.s	rx_p1			;buffer wrap?
	moveq	#0,d1			;shore did, do wrap
rx_p1:
	cmp.w	HEAD(a0),d1		;end of buffer?
	beq.s	rx_exit			;yes bump out
	move.w	d1,-(a7)        ;push index onto stack
	moveq	#1,d0
	bsr		buff_predict	;attempt to predict when buffer will fill
	move.w	(a7)+,d1		;pop index from stack
	move.b	RBA,d0			;get data from port
	move.b	d0,0(a1,d1)		;save data into buffer
	move	d1,TAIL(a0)		;store tail index
	addq.w	#1,NCHARS(a0)	;increment number of characters
rx_exit:
	rts

; tx_port takes a pointer to an rs232 port descriptor and takes data
; from the buffer and puts it into the transmit register

	XREF	_t_rec
_tx_port:
	btst.b	#2,SRA			;test status reg
	beq.s	tx_exit
	movea.l	#_t_rec,a0		;address of output record
	move.w	NCHARS(a0),d0
	bne.s	tout1			;no, continue processing
	bclr.b	#0,_imr_shad	;disable interrupt
	move.b	_imr_shad,IMR
	rts						;exit, done with interrupt

tout1:
	move.w	HEAD(a0),d0		;get head pointer
	movea.l	BUFFER(a0),a1	;address of data buffer
	addq.w	#1,d0			;i +EQU 1
	cmp.w	SIZE(a0),d0		;size of buffer exceeded?
	bcs.s	tout01
	moveq	#0,d0			;if yes, do a wrap
tout01:
	move.b	0(a1,d0),TBA	;put data into UART
	move	d0,HEAD(a0)		;save index
	subq.w	#1,NCHARS(a0)	;decrement number of characters
	btst.b	#2,SRA			;test the status flag again
	bne.s	_tx_port		;transmit not sated, do more
tx_exit:
	rts

;************************************************************************
; MIDI code uses port B.  This stuff needs to be as fast as posible
;************************************************************************

	XREF	_midi_r_rec

_rx_midi:
	movea.l	#_midi_r_rec,a0	;input port descriptor
	movea.l	BUFFER(a0),a1	;pointer to rs232 ring buffer
	move.w	TAIL(a0),d1		;index into rs232 ring buffer
	addq	#1,d1			;increment tail pointer
	cmp.w	SIZE(a0),d1		;i >EQU to current size?
	bcs.s	rmid_p1			;buffer wrap?
	moveq	#0,d1			;shore did, do wrap
rmid_p1:
	cmp.w	HEAD(a0),d1		;end of buffer?
	beq.s	rmid_exit		;yes bump out
	move.w	d1,-(a7)        ;push index onto stack
	moveq	#2,d0
	bsr.s	buff_predict	;attempt to predict when buffer will fill
	move.w	(a7)+,d1		;pop index from stack
	move.b	RBB,d0			;get data from port and clear interrupt
	move.b	d0,0(a1,d1)		;save data into buffer
	move	d1,TAIL(a0)		;store tail index
	addq.w	#1,NCHARS(a0)	;increment number of characters in buffer
rmid_exit:
	rts

;
; predict when the data buffer is going to be full
;
; a0 contains a pointer to the data structure for the sieral port
; d1 contains the current (next) tail pointer
; d0 contains bit to check in imr_shad
;
buff_predict:
	move.w	NCHARS(a0),d1	;get number of characters in buffer
	cmp.w	HIGH(a0),d1		;compare to high water mark
	blt.s	bp_xit          ;not about to overflow
	move.b	d0,BITRESET		;clear DSR line

;	bclr.b	d0,_imr_shad    ;	clear interrup enable bit
;	move.b	_imr_shad,IMR	;	clear ISR enable for UART ch B
bp_xit:
	rts


;
; transmit interrupt routine
;

	XREF	_midi_t_rec

_tx_midi:
	btst.b	#2,SRB			;test status reg
	beq.s	txm_exit
	movea.l	#_midi_t_rec,a0		;*	address of output record
	move.w	NCHARS(a0),d0
	bne.s	mout1			;no, continue processing
	bclr.b	#4,_imr_shad	;disable interrupt
	move.b	_imr_shad,IMR
	rts						;exit, done with interrupt

mout1:
	move.w	HEAD(a0),d0		;get head pointer
	movea.l	BUFFER(a0),a1	;	address of data buffer
	addq.w	#1,d0			; i +EQU 1
	cmp.w	SIZE(a0),d0		;	size of buffer exceeded?
	bcs.s	mout01
	moveq	#0,d0			;	if yes, do a wrap
mout01:
	move.b	0(a1,d0),TRB	;	put data into UART
	move	d0,HEAD(a0)		;	save index
	subq.w	#1,NCHARS(a0)	;decrement number of characters
	btst.b	#2,SRB			;test the status flag again
	bne.s	_tx_midi		;transmit not sated, do more
txm_exit:
	rts

;**********************************************************
; Misc interrupt routines
;**********************************************************

_break_b:
	link	a6,#0
	unlk	a6
	rts
_break_a:
	link	a6,#0
	unlk	a6
	rts
timer:
	jsr		_the_timer	;goto timer interrupt handler
	move.b	STOP_COUNTER,d0		/*	dummy read, clear interrupt	*/
	rts
_in_port_c:
	link	a6,#0
;	jsr		_port_change	;goto port change handler
	unlk	a6
	rts

	XREF	_p_error
	XDEF buss_error,address_error
	XDEF illegal,zero_div,chk_instr,trapv_instr
buss_error:
	moveq #2,d0
	bra.s	goto_error
address_error:
	moveq #3,d0
	bra.s	goto_error
illegal:
	moveq #4,d0
	bra.s goto_error
zero_div:
	moveq #5,d0
	bra.s goto_error
chk_instr:
	moveq #6,d0
	bra.s goto_error
trapv_instr:
	moveq #7,d0
	bra.s goto_error

	XDEF privilege,trace,line_a,line_f,reserved

privilege:
	moveq #8,d0
	bra.s goto_error
trace:
	moveq #9,d0
	bra.s goto_error
line_a:
	moveq #10,d0
	bra.s goto_error
line_f:
	moveq #11,d0
	bra.s goto_error
reserved:
	moveq #12,d0
	bra.s goto_error

	XDEF unitialized,spurious,autovector,trap_instr,user_vecs

unitialized:
	moveq #15,d0
	bra.s goto_error
spurious:
	moveq #24,d0
	bra.s goto_error
autovector:
	moveq #25,d0
	bra.s goto_error
trap_instr:
	moveq #32,d0
	bra.s goto_error
user_vecs:
	move.w #64,d0
goto_error:
	move	d0,-(a7)
	jsr	_p_error
	stop	#0				;just halt everything

