;***********************************************************************
;	Example startup code for the 68000 family to initialize RAM
;	and call the user function main().
;***********************************************************************
	XDEF	START,__brkp,__brksz
	XREF	STKTOP,DATA,_main
	XREF	MALLOC_SIZE,MALLOC_START

;***********************************************************************
;	Define vector area
;***********************************************************************
	SECTION vectors
__vectors	DS.L	256	;256 vectors at start of ram

;***********************************************************************
;	Define variables to track memory allocations for mbrk().
;***********************************************************************
	SECTION	data
__brkp	DC.L	MALLOC_START			; point to available memory
__brksz	DC.L	MALLOC_SIZE			; number of available bytes

;***************************************************************
;	The reset vector will point to this code (START).
;	By the time we get here, A7 will have been initialized.
;***************************************************************
	SECTION	code
START	MOVE.L	#0,A6			; end of stack frame chain

	;***********************************************************
	;		Zero out uninitialized RAM.
	;***********************************************************
	MOVE.L	#`BASE(ram),A1		; point A1 to base of region ram
	MOVE.L	#`SIZE(ram),D0		; set D0 to size of region ram
	LSR.L	#2,D0			; compute size in longs
	BRA	ZDBF			; enter a fast loop
  ZLP:	CLR.L	(A1)+			; clear four bytes at a time
  ZDBF:	DBF	D0,ZLP			; assumes `SIZE(ram) < 256K bytes

	;***********************************************************
	;		Initialize other RAM from ROM.
	;***********************************************************
	MOVE.L	#DATA,A0		; point A0 to ROM image of region data
	MOVE.L	#`BASE(data),A1		; point A1 to RAM base of region data
	MOVE.L	#`SIZE(data),D0		; set D0 to size of region data
	LSR.L	#2,D0			; compute size in longs
	BRA	IDBF			; enter a fast loop
  ILP:	MOVE.L	(A0)+,(A1)+		; move four bytes at a time
  IDBF:	DBF	D0,ILP			; assumes `SIZE(data) < 256K bytes
	;
	;***********************************************************
	;		Initialize RAM vectors from ROM
	;***********************************************************
	;
;	XREF	EXCEPTION_VECTORS
	MOVE.L	#exception_vecs,A0	;point A0 to ROM image of exception vector data
	MOVE.L	#`BASE(vectors),A1	;point A1 to RAM base of region vectors
	MOVE.L	#`SIZE(vectors),D0	;set D0 to size of region vectors
	LSR.L	#2,D0			;compute the size in longs
	BRA	EDBF			;enter a fast loop
ELP:	MOVE.L	(A0)+,(A1)+		;move four bytes at a time
EDBF:	DBF	D0,ELP
	;
	;***********************************************************
	;		Initialize memory allocator.
	;***********************************************************
;	 MOVE.L	#`BASE(malloc),D0	; address of malloc region
;	MOVE.L	#`SIZE(malloc),D1	; size of malloc region
;    .IF "ptrd"?"2"
;	SWAP	D0			; accommodate 2-byte C "pointers"
;    .ENDIF
;    .IF "long"?"2"
;	SWAP	D1			; accommodate 2-byte C "longs"
;    .ENDIF
;	MOVE.L	D0,__brkp		; variables referenced by mbrk()
;	MOVE.L	D1,__brksz
;
	;***********************************************************
	;		Invoke main() with no arguments.
	;***********************************************************
	JSR	_main			; any return value is "int" in D0
  DONE:	BRA	DONE			; infinite loop if main ever returns

;***********************************************************************
;	OTHER EXCEPTION VECTORS: to supervisor data space at address 8,
;	or 8 bytes beyond where the vector base register will point.
;	This table is commented out because no actual interrupt routines
;	are provided.
;***********************************************************************
	SECTION	vects
	XREF buss_error,address_error
	XREF illegal,zero_div,chk_instr,trapv_instr,privilege
	XREF trace,line_a,line_f,reserved,unitialized
	XREF spurious,autovector,trap_instr,user_vecs
	XREF _rs232_irq
exception_vecs:
	DC.L	STKTOP			; initial stack pointer
	DC.L	START			; initial execution address
	DC.L	buss_error,address_error		; 0x08
	DC.L	illegal,zero_div,chk_instr,trapv_instr	; 0x10
	DC.L	privilege,trace,line_a,line_f		; 0x20
	DC.L	reserved,reserved,reserved,unitialized	; 0x30
	DCB.L	8,reserved				; 0x40
	DC.L	spurious,autovector,autovector	; 0x60
#ifdef PYSYNTH
	DC.L	autoevector
#else
	DC.L	_rs232_irq
#endif
	DCB.L	4,autovector				; 0x70
	DCB.L	16,trap_instr				; 0x80
	DCB.L	16,reserved				; 0xc0
	DC.L	_rs232_irq				; 0x100
	DCB.L	191,user_vecs				; 0x104
