/*
** This is the timer routines
*/
#include <stdio.h>
#include "timer.h"
#include "rs232.h"

#define P_PORT 0xd0000

int ticks,time_ms;

extern char imr_shad;

typedef struct {
	int timer;
	int note;
} TIMER;

static int bitmap[8] = { 0x80,0x40,0x20,0x10,0x8,0x4,0x2,0x1};

static volatile TIMER r_timers[16];	/*	16 timers	*/
char  volatile pshad[4];

int setup_timer(int time,int note)
{
	/*
	** create a timer, puts pointer into pointer array and returns
	** handle to that timer, returns -1 if there are no more timers
	*/
	register int i;
	register TIMER *t;
	register int bit;
	register int port;

	for(i=0,t = r_timers;i<16;++i,++t)
	{
		if(t->note < 0)
		{
			Disable_Timer();
			t->timer = time;
			t->note = note;
			port = (note >> 3) & 3;	/* divide by 8	*/
			bit = note & 0x07;	/*	mod by 8	*/
			pshad[port] |= bitmap[bit];	/*	map bit	*/
			*((char *)(P_PORT + port)) = pshad[port];
			Enable_Timer();
			return(i);
		}
	}
	return(-1);	/*	no timer found	*/
}

void the_timer()
{
	static char flag;
	register int i;
	register volatile TIMER *t;
	register int port,bit;

	/*
	** this routine gets called from the interrupt service routine
	*/
	time_ms++;	/*	increment time	*/
	ticks--;	/*	decrement global ticker	*/
	t = r_timers;
	for(i=0;i<16;++i,++t)
	{
		if(t->note >= 0)	/*	if it points to something	*/
		{
			t->timer--;		/*	decrement it	*/
			if(t->timer == 0)	/*	timer is all done	*/
			{
				/*
				** clear bit in port
				*/
				port = (t->note >> 3) & 3;	/* divide by 8	*/
				bit = t->note & 0x07;	/*	mod by 8	*/
				pshad[port] &= ~bitmap[bit];	/*	map bit	*/
				*((char *)(P_PORT + port)) = pshad[port];
				t->note = -1;	/*	free up note	*/
			}
		}
	}
	/*
	** watch dog code
	*/
	if(flag)
	{
		flag = 0;
		*((char *)(UART+BIT_RESET_COMMAND)) = 0x10;
	}
	else
	{
		flag = 1;
		*((char *)(UART+BIT_SET_COMMAND)) = 0x10;
	}
}

void init_timer()
{
	register int dummy,i;
	register TIMER *t;

	t = r_timers;
	for(i=0;i<16;++i,++t)
		t->note = -1;		/*	initialize timer array	*/
	for(i=0;i<4;++i)
	{
    	pshad[i] = 0;
		*((char *)(P_PORT + i)) = pshad[i];
	}
	*((char *)(UART + COUNTER_UPPER_REG)) = 9;
	*((char *)(UART + COUNTER_LOWER_REG)) = 6; 	/*	10 mSEC timer default	*/
	acr_shad |= 0x70;
	*((char *)(UART + AUX_CONTROL_REG)) = acr_shad;
	imr_shad |= 0x08;	/*	enable bit for timer and input port change	*/
	*((char *)(UART + INTERRUPT_MASK_REG)) = imr_shad;	/*	enable timer	*/
	dummy = *((char *)(UART + START_COUNTER));
}

Disable_Timer()
{
	imr_shad &= ~0x08;	/*	enable bit for timer and input port change	*/
	*((char *)(UART + INTERRUPT_MASK_REG)) = imr_shad;	/*	enable timer	*/
}

Enable_Timer()
{
	imr_shad |= 0x08;	/*	enable bit for timer and input port change	*/
	*((char *)(UART + INTERRUPT_MASK_REG)) = imr_shad;	/*	enable timer	*/
}

void set_timer(unsigned val)
{
	register int dummy;

	*((char *)(UART + COUNTER_UPPER_REG)) = (char)(val >> 8);
	*((char *)(UART + COUNTER_LOWER_REG)) = (char)val; 	/*	10 mSEC timer default	*/
	dummy = *((char *)(UART + START_COUNTER));
}

